@@ -14,13 +14,16 @@
 # limitations under the License.
 
 import json
+import os
 import time
 try:
     # py3
+    import configparser
     from urllib.parse import urlencode
     from urllib.request import urlopen
 except ImportError:
     # py2
+    import ConfigParser as configparser
     from urllib import urlencode
     from urllib import urlopen
 
@@ -28,34 +31,42 @@ from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentClo
 from tencentcloud.common.common_client import CommonClient
 
 class Credential(object):
-    def __init__(self, secretId, secretKey, token=None):
+    def __init__(self, secret_id, secret_key, token=None):
         """Tencent Cloud Credentials.
 
         Access https://console.cloud.tencent.com/cam/capi to manage your
         credentials.
 
-        :param secretId: The secret id of your credential.
-        :type secretId: str
-        :param secretKey: The secret key of your credential.
-        :type secretKey: str
+        :param secret_id: The secret id of your credential.
+        :type secret_id: str
+        :param secret_key: The secret key of your credential.
+        :type secret_key: str
         :param token: The federation token of your credential, if this field
-                      is specified, secretId and secretKey should be set
+                      is specified, secret_id and secret_key should be set
                       accordingly, see: https://cloud.tencent.com/document/product/598/13896
         """
-        if secretId is None or secretId.strip() == "":
+        if secret_id is None or secret_id.strip() == "":
             raise TencentCloudSDKException("InvalidCredential", "secret id should not be none or empty")
-        if secretId.strip() != secretId:
+        if secret_id.strip() != secret_id:
             raise TencentCloudSDKException("InvalidCredential", "secret id should not contain spaces")
-        self.secretId = secretId
+        self.secret_id = secret_id
 
-        if secretKey is None or secretKey.strip() == "":
+        if secret_key is None or secret_key.strip() == "":
             raise TencentCloudSDKException("InvalidCredential", "secret key should not be none or empty")
-        if secretKey.strip() != secretKey:
+        if secret_key.strip() != secret_key:
             raise TencentCloudSDKException("InvalidCredential", "secret key should not contain spaces")
-        self.secretKey = secretKey
+        self.secret_key = secret_key
 
         self.token = token
 
+    @property
+    def secretId(self):
+        return self.secret_id
+
+    @property
+    def secretKey(self):
+        return self.secret_key
+
 
 class CVMRoleCredential(object):
     _metadata_endpoint = "http://metadata.tencentyun.com/latest/meta-data/"
@@ -134,6 +145,11 @@ class CVMRoleCredential(object):
             # maybe we should validate token to None as well
             pass
 
+    def get_credential(self):
+        if not self.secret_id or not self.secret_key or not self.token:
+            return None
+        return self
+
 
 class STSAssumeRoleCredential(object):
     """使用STSAssumeRoleCredential，制动role，
@@ -144,25 +160,20 @@ class STSAssumeRoleCredential(object):
     _version = '2018-08-13'
     _service = "sts"
 
-    def __init__(self, secret_id=None, secret_key=None, role_arn=None, role_session_name=None, duration_seconds=7200):
+    def __init__(self, secret_id, secret_key, role_arn, role_session_name, duration_seconds=7200):
         """
         :param secret_id: 接口调用凭证id
         :type secret_id: str
         :param secret_key: 接口调用凭证key
         :type secret_key: str
-
         https://cloud.tencent.com/document/api/1312/48197
-        :param role_arn: 角色的资源描述，上述链接RoleArn参数中有详细获取方式
+        :param role_arn: 角色的资源描述，参考官网文档 https://cloud.tencent.com/document/api/1312/48197 中 RoleArn 参数的描述。
         :type role_arn: str
         :param role_session_name: 临时会话名称，由用户自定义名称
         :type role_session_name: str
         :param duration_seconds: 获取临时凭证的有效期，默认7200s
         :type duration_seconds: int
         """
-
-        if None in [secret_id, secret_key, role_arn, role_session_name]:
-            raise TencentCloudSDKException("STSAssumeRoleCredential Parameter Error, "
-                                           "secret_id, secret_key, role_arn, role_session_name all required.")
         self._long_secret_id = secret_id
         self._long_secret_key = secret_key
         self._role_arn = role_arn
@@ -187,6 +198,16 @@ class STSAssumeRoleCredential(object):
         self._need_refresh()
         return self._tmp_secret_key
 
+    @property
+    def secret_id(self):
+        self._need_refresh()
+        return self._tmp_secret_id
+
+    @property
+    def secret_key(self):
+        self._need_refresh()
+        return self._tmp_secret_key
+
     @property
     def token(self):
         self._need_refresh()
@@ -216,13 +237,114 @@ class STSAssumeRoleCredential(object):
         params = {
             "RoleArn": self._role_arn,
             "RoleSessionName": self._role_session_name,
-            "DurationSeconds": self._duration_seconds
-
+            "DurationSeconds": self._duration_seconds,
         }
 
-        t_c = common_client.call_json("AssumeRole", params)
-        self._token = t_c["Response"]["Credentials"]["Token"]
-        self._tmp_secret_id = t_c["Response"]["Credentials"]["TmpSecretId"]
-        self._tmp_secret_key = t_c["Response"]["Credentials"]["TmpSecretKey"]
-        self._expired_time = t_c["Response"]["ExpiredTime"] - self._duration_seconds*0.9
+        rsp = common_client.call_json("AssumeRole", params)
+        self._token = rsp["Response"]["Credentials"]["Token"]
+        self._tmp_secret_id = rsp["Response"]["Credentials"]["TmpSecretId"]
+        self._tmp_secret_key = rsp["Response"]["Credentials"]["TmpSecretKey"]
+        self._expired_time = rsp["Response"]["ExpiredTime"] - self._duration_seconds * 0.9
 
+
+class EnvironmentVariableCredential():
+
+    def get_credential(self):
+        """Tencent Cloud EnvironmentVariableCredential.
+
+        Access https://console.cloud.tencent.com/cam/capi to manage your
+        credentials.
+
+        :param secret_id: The secret id of your credential, get by environment variable TENCENTCLOUD_SECRET_ID
+        :type secret_id: str
+        :param secret_key: The secret key of your credential. get by environment variable TENCENTCLOUD_SECRET_KEY
+        :type secret_key: str
+        """
+        self.secret_id = os.environ.get('TENCENTCLOUD_SECRET_ID')
+        self.secret_key = os.environ.get('TENCENTCLOUD_SECRET_KEY')
+
+        if self.secret_id is None or self.secret_key is None:
+            return None
+        if len(self.secret_id) == 0 or len(self.secret_key) == 0:
+            return None
+        return Credential(self.secret_id, self.secret_key)
+
+
+class ProfileCredential():
+
+    def get_credential(self):
+        """Tencent Cloud ProfileCredential.
+
+        Access https://console.cloud.tencent.com/cam/capi to manage your credentials.
+
+        default file position is "~/.tencentcloud/credentials" or "/etc/tencentcloud/credentials", it is ini format.
+        such as:
+        [default]
+        secret_id=""
+        secret_key=""
+
+        :param secret_id: The secret id of your credential.
+        :type secret_id: str
+        :param secret_key: The secret key of your credential.
+        :type secret_key: str
+        """
+        if os.path.exists(os.environ['HOME'] + "/.tencentcloud/credentials"):
+            file_path = os.environ['HOME'] + "/.tencentcloud/credentials"
+        elif os.path.exists("/etc/tencentcloud/credentials"):
+            file_path = "/etc/tencentcloud/credentials"
+        else:
+            file_path = ""
+        if file_path:
+            # loads config
+            conf = configparser.ConfigParser()
+            conf.read(file_path)
+            ini_map = dict(conf._sections)
+            for k in dict(conf._sections):
+                option = dict(ini_map[k])
+                for key, value in dict(ini_map[k]).items():
+                    option[key] = value.strip()
+                ini_map[k] = option
+            if "default" in ini_map:
+                client_config = ini_map.get("default")
+                self.secret_id = client_config.get('secret_id', None)
+                self.secret_key = client_config.get('secret_key', None)
+                self.role_arn = client_config.get('role_arn', None)
+        else:
+            self.secret_id = None
+            self.secret_key = None
+            self.role_arn = None
+
+        if self.secret_id is None or self.secret_key is None:
+            return None
+        if len(self.secret_id) == 0 or len(self.secret_key) == 0:
+            return None
+        return Credential(self.secret_id, self.secret_key)
+
+
+class DefaultCredentialProvider(object):
+    """Tencent Cloud DefaultCredentialProvider.
+
+    DefaultCredentialProvider will search credential by order EnvironmentVariableCredential ProfileCredential
+    and CVMRoleCredential.
+    """
+
+    def __init__(self):
+        self.cred = None
+
+    def get_credentials(self):
+        if self.cred is not None:
+            return self.cred
+
+        self.cred = EnvironmentVariableCredential().get_credential()
+        if self.cred is not None:
+            return self.cred
+
+        self.cred = ProfileCredential().get_credential()
+        if self.cred is not None:
+            return self.cred
+
+        self.cred = CVMRoleCredential().get_credential()
+        if self.cred is not None:
+            return self.cred
+
+        raise TencentCloudSDKException("ClientSideError", "no valid credentail.")
